from zope import schema, interface
from zope.app.securitypolicy.interfaces import IPrincipalRoleManager
from zope.app.form.browser.add import AddView
from zope.app.form.interfaces import WidgetsError
from zope import component
from zope.app import zapi
from zope.app.container.interfaces import INameChooser
from zope.formlib import form
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
import zope.event
from zope.interface.common import idatetime
import datetime
import pytz
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.security.proxy import removeSecurityProxy
from zope.location.location import locate
from zope.app.securitypolicy.interfaces import IPrincipalPermissionManager
from zope.security import canAccess

from timeclock import interfaces
from timeclock import work
from timeclock.app import getTimeClockApplication
from timeclock.browser import app
from timeclock.browser.widget import PersonWidget


class WorkLogAddView(AddView):
    def add(self, content):
        result = self.context.add(content)
        principal = self.person_widget.getInputValue()
        ppm = IPrincipalPermissionManager(content)
        ppm.grantPermissionToPrincipal('timeclock.edit',principal.login)
        return result

    def nextURL(self):
        return zapi.absoluteURL(self.context.__parent__, self.request)


class WorkLogAddForm(form.AddForm):

    form_fields = form.Fields(interfaces.IWorkLog)
    form_fields['person'].custom_widget = PersonWidget

    label="Add a Work Log"

    def setUpWidgets(self, ignore_request=False):
        import pdb; pdb.set_trace()
        self.widgets = form.setUpInputWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            ignore_request=ignore_request,
            )

    def create(self, data):
        worklog = work.WorkLog()
        if form.applyChanges(worklog, self.form_fields, data, self.adapters):
            return worklog

    def nextURL(self):
        return zapi.absoluteURL(self.context, self.request)


class WorkLogEditForm(app.TimeClockEditForm):

    form_fields = form.Fields(interfaces.IWorkLog).select('title','hourlyPay','reportHeader')
    label="Edit a Work Log"

class WorkLogContainerWidget(object):

    def submitted(self):
        for submitted in self.context.submitted.values():
            if submitted.status == "Newly Submitted":
                yield submitted

    def canAccess(self):
        return canAccess(self.context, 'submitted')

class WorkLogContainerForm(app.ContainerEditForm):

    title = u"Work Logs"

    @property
    def canDelete(self):
        return canAccess(self.context, '__delitem__')

class WorkLogEntryEditForm(app.TimeClockEditForm):

    form_fields = form.Fields(interfaces.IWorkLogEntry).omit('cost','totalTime')
    label="Edit a Work Log"

    def setUpWidgets(self, ignore_request=False):
        super(WorkLogEntryEditForm, self).setUpWidgets(ignore_request)
        self.widgets['workDone'].height = 5
        self.widgets['learned'].height = 5


class WorkLogFormSchema(interface.Interface):

    startDate = schema.Datetime(title=u"Start Date", required=False)
    endDate = schema.Datetime(title=u"End Date", required=False)

class WorkLogForm(form.Form):

    form_fields = form.Fields(WorkLogFormSchema)
    template = ViewPageTemplateFile('worklog.pt')
    clockIn = form.Action('Clock In', success='handle_clock_in',
                    condition=lambda self, action: not self.context.clockedIn)
    clockOut = form.Action('Clock Out', success='handle_clock_out',
                    condition=lambda self, action: self.context.clockedIn)
    clockActions = form.Actions(clockIn, clockOut)

    otherActions = form.Actions(
        form.Action('Search', success='handle_search_action'),
        form.Action('Submit these Entries', success='handle_submit_entries'))
    actions = clockActions + otherActions

    searching = False

    def update(self):
        if self.context.clockedIn:
            self.status = "Clocked in since: %s" % self.context.clockedInTime.strftime('%x %X')
        super(WorkLogForm, self).update()
    def setUpWidgets(self, ignore_request=False):
        if self.searching:
            data = dict(startDate=self.startDate,
                        endDate=self.endDate)
        else: data = {}
        self.widgets = form.setUpWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            data=data,
            ignore_request=ignore_request
            )

    def handle_submit_entries(self, action, data):
        self.handle_search_action(action, data)
        entries = self.getEntries()
        if len(entries) == 0:
            self.status = "There were no entries to submit."
            return
        if self.searching:
            self.context.submitEntries(self.startDate, self.endDate)
            self.status = "Submitted entries between %s and %s" % (self.startDate.strftime('%x'),
                                                                   self.endDate.strftime('%x'))
        else:
            self.context.submitEntries()
            self.status = "Submitted all entries."

    def handle_search_action(self, action, data):
        self.startDate = data.get('startDate')
        self.endDate = data.get('endDate')
        if self.startDate and self.endDate:
            self.searching = True
            self.status = "Looking at entries between %s and %s" % (self.startDate.strftime('%x'),
                                                                    self.endDate.strftime('%x'))

    def handle_clock_in(self, action, data):
        self.context.clockedIn = True
        self.context.clockedInTime = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)

    def handle_clock_out(self, action, data):
        self.context.clockedIn = False

        workLogEntry = work.WorkLogEntry()
        workLogEntry.startTime = self.context.clockedInTime
        workLogEntry.endTime = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        zope.event.notify(ObjectCreatedEvent(workLogEntry))

        date = '%s-%s-%s' % self.context.clockedInTime.isocalendar()
        name = INameChooser(self.context).chooseName(date, workLogEntry)
        self.context[name] = workLogEntry
        self.request.response.redirect(zapi.absoluteURL(workLogEntry, self.request)+'/@@edit.html')

    def getEntries(self):
        entries = list(self.context.values())
        entries.sort(cmp=lambda a,b: cmp(b.startTime,a.startTime))
        if self.searching:
            entries = [entry for entry in entries
                       if self.startDate <= entry.startTime <= self.endDate]
        return entries

    @property
    def entries(self):
        entries = self.getEntries()
        self.totalTime = sum([e.totalTime for e in entries])
        self.totalTime = '%i:%02i' % (self.totalTime, self.totalTime*60%60)
        self.totalCost = '%.2f' % sum([e.cost for e in entries])
        for entry in entries:
            data = {}
            data['object'] = entry
            data['startTime'] = entry.startTime.strftime('%x')
            data['endTime'] = entry.endTime.strftime('%x')
            netTime = entry.totalTime
            data['timeWorked'] = '%i:%02i' % (netTime,netTime*60%60)
            data['workDone'] = entry.workDone
            data['learned'] = entry.learned
            data['cost'] = '%.2f' % entry.cost
            yield data

class WorkLogReportView(WorkLogForm):
    template = ViewPageTemplateFile('worklogreport.pt')
    form_fields = form.Fields()
    actions = form.Actions()

    def __init__(self, context, request):
        super(WorkLogReportView, self).__init__(context, request)
        self.time = datetime.datetime.utcnow().strftime("%x %X")

class SubmittedWorkLogReportView(WorkLogReportView):

    @form.action("Processed",
                 condition=lambda self, action: self.context.status == "Newly Submitted")
    def handle_processed(self, action, data):
        self.context.status = "Processed"

    @form.action("Payment Recieved",
                 condition=lambda self, action: self.context.status == "Processed")
    def handle_payment_recieved(self, action, data):
        self.context.status = "Payment Recieved"


class WorkLogEntryDisplayForm(form.DisplayForm):
    form_fields = form.Fields(interfaces.IWorkLogEntry)

    label = "Work Log Entry"


class SubmittedWorkLogContainerForm(app.ContainerDisplayForm):

    template = ViewPageTemplateFile('submittedcontainer.pt')

    title = u"Submitted Work Logs"


class WorkLogGraphView(object):

    def data(self):
        times = [entry.totalTime for entry in sorted(self.context.values())]
        return dict(times=times)


class WorkLogExportView(object):

    def __call__(self):
        result = []
        result.append(["Title:", self.context.title])
        result.append(["Employee:", self.context.person.title])
        result.append(["Hourly Pay:",'$'+str(self.context.hourlyPay)])
        result.append(["Start Time", "End Time", "Work Done", "Learned", "Total Time (hours)", "Cost"])
        for entry in self.context.values():
            result.append([entry.startTime.strftime("%x %X"),
                           entry.endTime.strftime("%x %X"),
                           str(entry.workDone).replace(',',''),
                           str(entry.learned).replace(',',''),
                           entry.totalTime,
                           entry.cost])
        result.append(["","","","",sum([e.totalTime for e in self.context.values()]),
                                   sum([e.cost for e in self.context.values()])])
        final = ""
        for line in result:
            for item in line:
                final += str(item)+','
            final += '\n'
        return final
