import zope.interface
from zope.app.container.interfaces import INameChooser
from zope.interface import implements
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import Contained
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.app.securitypolicy.interfaces import IPrincipalPermissionManager
from zope.publisher.interfaces import IPublishTraverse, NotFound
from zope.component import subscribers, queryAdapter, queryMultiAdapter
from datetime import datetime
from zope.schema.fieldproperty import FieldProperty
from interfaces import IWorkLogEntry, IWorkLog, IWorkLogEntryContained, IWorkLogContained, IWorkLogContainer
from pytz import utc
from person import Person
from persistent import Persistent
from zope.location.location import LocationProxy, locate
from zope.app.container.interfaces import IContained
import interfaces

class WorkLogEntry(Contained, Persistent):
    """An implementations of IWorkLogEntry."""
    implements(IWorkLogEntry,
               IWorkLogEntryContained,
               IAttributeAnnotatable)

    workDone = FieldProperty(IWorkLogEntry['workDone'])
    learned = FieldProperty(IWorkLogEntry['learned'])
    startTime = FieldProperty(IWorkLogEntry['startTime'])
    endTime = FieldProperty(IWorkLogEntry['endTime'])

    @property
    def totalTime(self):
        dt = self.endTime-self.startTime
        return dt.days*24+dt.seconds/60.**2

    @property
    def cost(self):
        hourlyPay = self.__parent__.hourlyPay or 0.
        return hourlyPay*self.totalTime

    def __cmp__(self, other):
        return cmp(self.startTime, other.startTime)


class WorkLog(BTreeContainer):
    """Container of IWorkLogEntries."""	
    implements(IWorkLog, IWorkLogContained,
               IAttributeAnnotatable)

    hourlyPay = FieldProperty(IWorkLog['hourlyPay'])
    title = FieldProperty(IWorkLog['title'])
    reportHeader = FieldProperty(IWorkLog['reportHeader'])
    submitted = FieldProperty(IWorkLog['submitted'])
    clockedIn = False
    clockedInTime = None

    _person = None

    def __init__(self):
        super(WorkLog, self).__init__()
        self.submitted = SubmittedWorkLogContainer()
        locate(self.submitted, self, 'submitted')
        self.hourlyPay = 0.

    def submitEntries(self, startDate=None, endDate=None):
        submitted = SubmittedWorkLog(self)
        if startDate != None and endDate != None:
            for key,value in list(self.items()):
                if startDate <= value.startTime <= endDate:
                    submitted[key] = value
                    locate(value, submitted, key)
                    del self[key]
        else:
            for key,value in list(self.items()):
                submitted[key] = value
                locate(value, submitted, key)
                del self[key]
        temp = [s.startTime for s in submitted.values()]
        earliest = min(temp)
        latest = max(temp)
        name = '%s-%s' % (earliest.strftime('%Y.%m.%d'),latest.strftime('%Y.%m.%d'))
        name = INameChooser(self.submitted).chooseName(name, submitted)
        self.submitted[name] = submitted
        locate(submitted, self.submitted, name)

    def getPerson (self):
        return self._person

    def setPerson (self, person):
        ppm = IPrincipalPermissionManager(self)
        ppm.grantPermissionToPrincipal('timeclock.edit',person.login)
        print 'granted timeclock.edit to',person.login,'on',self.title
        if self._person:
            ppm.unsetPermissionForPrincipal('timeclock.edit',
                                            self._person.login)
        self._person = person

    person = property(getPerson, setPerson)

    @property
    def totalTime(self):
        return sum([e.totalTime for e in self.values()])

    @property
    def totalCost(self):
        return sum([e.cost for e in self.values()])


class SubmittedWorkLog(BTreeContainer, Contained):
    """Container for submitted work log entries."""
    implements(interfaces.ISubmittedWorkLog, IContained)

    workLog = FieldProperty(interfaces.ISubmittedWorkLog['workLog'])
    status = FieldProperty(interfaces.ISubmittedWorkLog['status'])

    def __init__(self, workLog):
        super(SubmittedWorkLog, self).__init__()
        self.workLog = workLog
        self.status = "Newly Submitted"

    def __getattr__(self, name):
        marker = object()
        if name == 'title':
            return self.title
        if name == 'workLog':
            return self.workLog
        if name == 'totalTime':
            return self.totalTime
        if name == 'totalCost':
            return self.totalCost
        v = getattr(self.workLog, name, marker)
        if v is marker:
            raise AttributeError
        return v

    totalTime = WorkLog.totalTime
    totalCost = WorkLog.totalCost

    @property
    def title(self):
        entries = sorted(self.values())
        start = entries[0].startTime
        end = entries[-1].startTime
        title = '%s (%s to %s)' % (self.workLog.title, start.strftime('%x'),end.strftime('%x'))
        return title


class SubmittedWorkLogContainer(BTreeContainer, Contained):
    """Container of SubmittedWorkLogs."""
    implements(interfaces.ISubmittedWorkLogContainer)


class WorkLogContainer(BTreeContainer, Contained):
    """Container of IWorkLogs."""
    implements(IWorkLogContainer, IContained,
               IAttributeAnnotatable)


class AttributeTraverser(object):
    """Submitted Pluggable Traverser."""

    implements(IPublishTraverse)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        # 1. Look at all the traverser plugins, whether they have an answer.
        try:
            return LocationProxy(getattr(self.context, name), self.context, name)
        except:
            pass

        try:
            return self.context[name]
        except:
            pass

        view = queryMultiAdapter((self.context, request), name=name)
        if view is not None:
            return view

        raise NotFound(self.context, name, request)
